/*
 * Copyright (c) 1988, 1990 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Scott R. Presnell (srp@cgl.ucsf.edu)
 *
 */
#ifndef lint
static  char rcsid[] =
    "@(#)$Header: pcap-snoop.c,v 1.12 90/12/04 17:13:24 srp Exp $ (UCSF)";
#endif

#include <stdio.h>
#include <netdb.h>
#include <ctype.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>

#include <net/raw.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/if_ether.h>
#include <netinet/ip_var.h>
#include <netinet/udp.h>
#include <netinet/udp_var.h>
#include <netinet/tcp.h>
#include <netinet/tcpip.h>
#include <net/bpf.h>

#include "interface.h"

   
#define ETHERHDRPAD	  RAW_HDRPAD(sizeof(struct ether_header))
   
struct etherpacket {
	struct snoopheader	  snoop;
	char			  pad[ETHERHDRPAD];
	struct ether_header	  ether;
	char			  data[ETHERMTU];
};

#define SOCKBUFSIZ	4000

void
readloop(cnt, if_fd, fp, printit)
	int cnt;
	int if_fd;
	struct bpf_program *fp;
	void (*printit)();
{
	struct etherpacket ep;
	struct bpf_insn *fcode;
	int cc, i;

	fcode = fp->bf_insns;
	i = 0;

	while ((cc = read(if_fd, (char *) &ep, sizeof(ep) )) > 0) {

		int caplen;
		
		caplen = ep.snoop.snoop_packetlen;

		if (caplen > snaplen)
			caplen = snaplen;
		if (bpf_filter(fcode, &(ep.ether),
			       ep.snoop.snoop_packetlen, caplen)) {
#if 0
			printf("%d %d %d ", 
			       snaplen, caplen, ep.snoop.snoop_packetlen);
#endif
			if (cnt >= 0 && --cnt < 0)
				goto out;

			(*printit)(&(ep.ether), &(ep.snoop.snoop_timestamp),
				   ep.snoop.snoop_packetlen, caplen);
		}
	}
	perror("read");
	exit(-1);

out:
	wrapup(if_fd);
}

wrapup(fd)
	int fd;
{
	close(fd);
}

int
initdevice(device, pflag, linktype)
	char *device;
	int pflag;
	int *linktype;
{
	struct sockaddr_raw sr;
	struct snoopfilter sf;
	int cc = SOCKBUFSIZ;
	int on = 1;
	int if_fd;

	/*
	 * Open a raw socket for snooping.
	 */
	if ((if_fd = socket(AF_RAW, SOCK_RAW, RAWPROTO_SNOOP)) < 0) {
		perror("socket");
		exit(-1);
	}

	/*
	 * If the family is not set (not specified in manual page), bind
	 * fails due to unsupported address family.
	 */
	sr.sr_family = AF_RAW;

	/*
	 * If we want a named device, name it, otherwize zero it.
	 */
	if (device == NULL)
		bzero(sr.sr_ifname, RAW_IFNAMSIZ);
	else
		strncpy(sr.sr_ifname, device, RAW_IFNAMSIZ);

	sr.sr_port = 0;


	if (bind(if_fd, (struct sockaddr *) &sr, sizeof (sr)) < 0) {
		perror("bind");
		exit(1);
	}

	/*
	 * Zero the filter.
	 */
	bzero((char *) &sf, sizeof sf);

	/*
	 * Unset the promiscuious flag if asked...
	 */
	if (pflag)
		sf.sf_promisc = 0;


	if (ioctl(if_fd, SIOCADDSNOOP, (struct snoopfilter *) &sf) < 0) {
		perror("ioctl: adding snoop filter");
		exit(1);
	}

	setsockopt(if_fd, SOL_SOCKET, SO_RCVBUF, (char *) &cc, sizeof cc);

	if (ioctl(if_fd, SIOCSNOOPING, &on) < 0) {
		perror("ioctl: turn on snoop");
		exit(1);
	}

	/*
	 * I don't know what other devices are supported under snoop(7)
	 */
	*linktype = DLT_EN10MB;

	return if_fd;
}
